#pragma once

#include <cstddef>

// these are generated by ../../misc/xxd.py

// ../doc/murphi2murphi.1
extern const unsigned char doc_murphi2murphi_1[];
extern const size_t doc_murphi2murphi_1_len;
